<?php
/**
 * Ulyxes server side script to serve data to clients
 *
 * This script is invoked by httprequest (AJAX) calls.
 * The POST or GET parameters of the request are:
 * type=points - get horizontal coordinates of the targets
 * type=buildings - get buildings' polygons
 * type=dates - get min, max date from database
 *   format - format specification for returned dates
 * type=survey - get settlement data within a period of time
 * 	 point_names= coma separated list of point names
 * 	 start_date
 * 	 end_date
 *
 * Sample URLs:
 *   http://localhost/ulyxes/getdata.php?type=points
 *   http://localhost/ulyxes/getdata.php?type=buildings
 *   http://localhost/ulyxes/getdata.php?type=dates&format=mm/dd/yy
 *   http://localhost/ulyxes/getdata.php?type=dates&format=mm/dd/yy
 *   http://localhost/ulyxes/getdata.php?type=survey&point_names=620
 *   
 * return JSON string with the requested data
 * in case of a points request:
 *  'xy': [{'point_name': '1230' , 'lng': '19.054391' , 'lat': '47.481882'},
 *         {'point_name': '1743' , 'lng': '19.056075' , 'lat': '47.481518'}]
 *
 * Suggested database schema
 *	points - WGS84 coordinates of points
 *		columns: point_name, geom(point)
 *	results - settlement of points
 *		columns: point_name, date, move
 *	buildings - additional polygons on map
 *		columns: gid, name, geom(polygon)
 *
 * @author Zoltan Siki <siki (at) agt.bme.hu>
 * @package Ulyxes
 */

include_once("config.php"); //external configuration file

if (! isset($_REQUEST["type"])) {
	die("Missing type parameter");
}
//connect to the database
$conn = pg_connect ("host=$dbhost dbname=$dbname user=$user password=$pass");
if (! $conn)
	die ("Connection failed.");

switch ($_REQUEST["type"]) {
// create JSON string, array of point_name longitude latitude
// alternative solution get data from WFS server
case "points" : $sql="SELECT point_name,ST_X(geom) as lng,ST_Y(geom) as lat FROM points ORDER BY point_name";
	$resource = @pg_query ($sql);
	if (! $resource) die ("Error");
	$buf = "{'xy':\n["; // start an array
	// go through all points
	while (($r=pg_fetch_array($resource,NULL,PGSQL_ASSOC))) {
		$buf .= "{'point_name': '" . $r["point_name"] . "' ,
			'lng': '" . $r["lng"] . "' , 'lat': '" . $r["lat"] . "'},\n";
	}
	echo trim($buf, ",\n");
	echo "\n]\n}";
	break;

// create JSON string of buildings polygons
// alternative solution get dat from WFS/WMS server
case "buildings" : $sql="SELECT name, ST_AsText(geom) as g, gid, ST_NPoints(geom) as n FROM buildings";
	$resource=@pg_query ($sql);
	if (! $resource) die ("Error");
	$buf = "{'buildings':\n[";
	while (($r=pg_fetch_array($resource,NULL,PGSQL_ASSOC))) {
		// process WKT polygon description
		$p = preg_split('/,/',trim(substr($r["g"], strpos($r["g"], "((")), "()"));
		$buf .= "{'name': '" . $r["name"] . "' , 'points': [";
		for ($i=0; $i < $r["n"]; $i++) {
			$pi = preg_split('/ /', $p[$i]);
			//from WKT, delimited with space
			$buf .= "{'lng': '" . $pi[0] . "' , 'lat': '" . $pi[1] . "'},\n";
		}
		$buf = trim($buf, ",\n");
		$buf .= "\n]\n},";
	}
	echo trim($buf, ",\n");
	echo "\n]\n}";
	break;

// create JSON string of settelment data
case "survey": $point_names = preg_split('/,/', $_REQUEST["point_names"]);
	$buf = "{'survey':\n[";
	for ($i=0 ; $i < count($point_names) ; $i++) {
		$sql="SELECT point_name, date, move FROM results WHERE point_name='" .
			$point_names[$i] . "'";
		if (!empty($_REQUEST["start_date"]) && !empty($_REQUEST["end_date"])) {
			$sql .= " AND date BETWEEN '" . $_REQUEST["start_date"] .
				"' AND '" . $_REQUEST["end_date"] . "'";
		}
		$sql .= " ORDER BY date";
		$resource = @pg_query($sql);
		if (! $resource) die ("Error");
		$buf .= "{'point_name': '" . $point_names[$i] . "' , 'data': [\n";
		while (($r=pg_fetch_array($resource,NULL,PGSQL_ASSOC))) {
			$buf .= "{'dt': '" . $r["date"] . "' , 'movement': '" . $r["move"] . "'},\n";
		}
		$buf = trim($buf, ",\n");
		$buf .= "\n]\n},\n";
	}
	$buf = trim($buf, ",\n");
	$buf .= "\n]\n}"; //json ends
	echo $buf;
	break;

// get min, max dates
case "dates": $sql = "SELECT to_char(min(date),'" . $_REQUEST["format"] . "') AS sd, to_char(max(date),'" . $_REQUEST["format"] . "') AS ed FROM results";
		$resource = @pg_query($sql);
		$r=pg_fetch_array($resource,NULL,PGSQL_ASSOC);
		$buf = "{ 'sd':'" . $r["sd"] . "', 'ed':'" . $r["ed"] . "'}";
		echo $buf;
		break;

default: die ("Incorrect parameters.");
}
pg_close();
?>
