<?php
/**
 * Ulyses demo application using OpenStreetMap, OpenLayers, jQuery and 
 * WMS/WFS server
 *
 * GPL v2.0 license
 * Demo application page width OpenLayers & WMS/WFS server
 * Optional query string parameters:
 *		lang - interface language en/hu, default en
 *		width - map width pixels, default 500
 *		height - map height pixels, default 500
 *
 * @author Zoltan Siki <siki (at) agt.bme.hu>
 *
 */

// load configuration & defaults
include_once("config.php");
// prefer query string lang parameter
if (isset($_REQUEST["lang"])) {
	if (array_search($_REQUEST["lang"], $langs)) {
		$lang = $_REQUEST["lang"];
	}
}
if (! isset($lang)) {
	$lang = "en";
}

// load language dependent messages
include_once("msg_" . $lang . ".php");

// set map widget size
if (isset($_REQUEST["width"])) {
	$width = $_REQUEST["width"];
}
if (! isset($width)) {
	$width = 500;
}
if (isset($_REQUEST["height"])) {
	$height = $_REQUEST["height"];
}
if (! isset($height)) {
	$height = 500;
}
?>

<html>
<head>
<meta http-equiv="content-type" content="text/html; charset=UTF-8"/>

<title><?php echo $msg["title"];?></title>
<!-- jquery/jquery-ui/flot chart plot/openLayers -->
<?php
echo '<link rel="stylesheet" type="text/css" href="http://' . $host . '/jquery_ui/css/ui-lightness/jquery-ui-1.8.22.custom.css">';
echo '<script src="http://' . $host . '/jquery_ui/js/jquery-1.7.2.min.js"></script>';
echo '<script src="http://' . $host . '/jquery_ui/js/jquery-ui-1.8.22.custom.min.js"></script>';
echo '<script language="javascript" type="text/javascript" src="http://' . $host . '/jquery_ui/flot/jquery.flot.min.js"></script>';
echo '<script language="javascript" type="text/javascript" src="http://' . $host . '/jquery_ui/flot/jquery.flot.crosshair.min.js"></script>';
echo '<script src="http://' . $host . '/OpenLayers/OpenLayers.js"></script>';
?>
<!-- locale javascript messages -->
<script type="text/javascript">
	var js_msg = new Array();
	var js_key = new Array();

// generating key and message javascript arrays for 
// multiligual interface from server side php array
<?php
	$i = 0;
	foreach ($js_msg as $key => $value) {
		echo "js_key[" . $i . "] = '" . $key . "';\n";
		echo "js_msg[" . $i . "] = '" . $value . "';\n";
		$i++;
	}
	echo "tickDiv = $tickDiv;";
	echo "buildings = $buildings;";
	echo "ulyxesHost = '$host';";
?>

// set date format & regexp for check
<?php
	if ($lang == "hu") {
		echo "dateReg = /^20[0-9]{2}-[0-1][0-9]-[0-3][0-9]$/;";
		echo "dFormat = 'yy-mm-dd';";
		echo "pgFormat = 'yyyy-mm-dd';";
	} else {
		echo "dateReg = /^[0-1][0-9]\/[0-3][0-9]\/20[0-9]{2}$/;";
		echo "dFormat = 'mm/dd/yy';";
		echo "pgFormat = 'mm/dd/yyyy';";
	}
?>

var bgcol = '#fbfad6';	// background color for form
var map;				// map object for openlayers
var points;				// layer with our points

/* remove leading and ending "c" characters from string */
String.prototype.trim = function (c) {
    var r = new RegExp("^"+c+"+|"+c+"+$");
    return this.replace(r, "");
}

// return message based on key
// DOESN'T WORK IN IE6!
function getMsg(key) {
	var k = js_key.indexOf(key);
	if (k < 0) return "Missing string!";
	return js_msg[k];
}

function chg() {
	var u = "<?php echo $_SERVER['PHP_SELF']; ?>";
	var s = $("#size").val().split(",");
	window.location = u + "?lang=" + $("#lang").val() +
		'&width=' + s[0] + '&height=' + s[1];
}

/* color array for points and graphs */
var cols = ["blue", "red", "green", "yellow"];
var selectPoint;	// select control for points layer
var popupFeature;	// feature with popup bubble

// initialization
$(document).ready(function() {
	// date pickers & default values
	$("#sd").datepicker( {dateFormat: dFormat, defaultDate: "-30"} );
	$("#ed").datepicker( {dateFormat: dFormat} );
	$("#t0").css("background-color", bgcol);
	// init OSM map & add points layer
	InitMap();
	// fill point list
	GetPoints();
	// get min/max dates from database
	GetDates();
	// reset form
	$("#res").click(function(e) {
		$("#tbl").html("");
		$("#queryform").find(":input").each(function() {
			switch(this.type) {
				case 'password':
				case 'select-multiple':
				case 'select-one':
				case 'text':
				case 'text-area':
					$(this).val("");
					break;
				case 'checkbox':
				case 'radio':
					this.checked = false;
			}
		});
	});
	$("#ini").click(function(e) {
		map.setCenter(new OpenLayers.LonLat(<?php echo $lon; ?>, <?php echo $lat; ?>), <?php echo $zoom; ?>);
	});
	$("#submit").click(function(e) {
		if (! CheckForm()) return;
		// collect form parameters
		var points = "";
		$("#pn option:selected").each(function() {
			points += $(this).text() + ",";
		});
		points = points.trim(",");
		var inData = { type: "survey",
			point_names : points,
			start_date: $("#sd").val(),
			end_date: $("#ed").val()
		}
		if ($("#tab").attr('checked')) {
			AddTable(inData);
		} else if ($("#cgra").attr('checked')) {
			AddLineChart(inData);
		}
	});

	$("#lang").change(function(e) { 
		chg();
	});
	$("#size").change(function(e) {
		chg();
	});
});

function InitMap() {
	google_mercator = new OpenLayers.Projection("EPSG:900913");
	WGS84 = new OpenLayers.Projection("EPSG:4326");
	map = new OpenLayers.Map('map', {
		projection: google_mercator
	});
	map.addControl(new OpenLayers.Control.LayerSwitcher());
	map.addControl(new OpenLayers.Control.OverviewMap());
	map.addControl(new OpenLayers.Control.KeyboardDefaults());
	map.addControl(new OpenLayers.Control.MousePosition());

// set up layers
	// OSM background
	map.addLayer(new OpenLayers.Layer.OSM());
	// extra polygons for buildings
	map.addLayer(new OpenLayers.Layer.Vector('buildings', {
		protocol: new OpenLayers.Protocol.WFS({
		url: 'http://' + ulyxesHost + '/cgi-bin/mapserv?map=/home/html/szonja/ol.map',
		featureType: 'buildings'}),
		visibility: buildings,
		strategies: [new OpenLayers.Strategy.Fixed()]
	}));
	// deformation analysis points
	points = new OpenLayers.Layer.Vector('points', {
		protocol: new OpenLayers.Protocol.WFS({
			url: 'http://' + ulyxesHost + '/cgi-bin/mapserv?map=/home/html/szonja/ol.map',
			featureType: 'points'}),
		strategies: [new OpenLayers.Strategy.Fixed()]
	});
	map.addLayer(points);
	// enable selection on points layer
	selectPoint = new OpenLayers.Control.SelectFeature(
		points,
		{
			multiple: false,
			toggle: true,
			multipleKey: 'shiftKey'
		}
	);
	// set up point styles
	var pStyle = new OpenLayers.Style({
		'cursor': 'pointer',
		'fillColor': '#787878',
		'fillOpacity': .8,
		'pointRadius': 5,
		'fontColor': '#0000ff',
		'fontSize': 12,
		'label': '${point_name}',
		'labelAlign': 'lb',
		'labelXOffset': 6,
		'labelYOffset': 6
	});
	var pSelectedStyle = new OpenLayers.Style({
		'cursor': 'pointer',
		'fillColor': '#ff0000',
		'fillOpacity': .8,
		'pointRadius': 7,
		'fontColor': '#0000ff',
		'fontSize': 16,
		'label': '${point_name}',
		'labelAlign': 'lb',
		'labelXOffset': 8,
		'labelYOffset': 8
	});
	var pStyleMap = new OpenLayers.StyleMap({
		'default': pStyle,
		'select': pSelectedStyle
	});
	points.styleMap = pStyleMap;

	map.addControl(selectPoint);
	selectPoint.activate();
	// set up select handlers
	points.events.register('featureselected', this, PointSelected);
	points.events.register('featureunselected', this, PointUnselected);
	map.setCenter(new OpenLayers.LonLat(<?php echo $lon; ?>,
		<?php echo $lat; ?>), <?php echo $zoom; ?>);
}

function onPopupClose(event) {
	selectPoint.unselect(popupFeature);
}

function PointSelected(event) {
//	alert(event.feature.attributes.point_name);
	popupFeature = event.feature;
	var attr = '<div><b>' + popupFeature.data.point_name + '</b><br />' +
		'x: ' + Math.round(popupFeature.geometry.x) + '<br />' +
		'y: ' + Math.round(popupFeature.geometry.y) + '<br />' +
		'<a href="' + popupFeature.data.point_name + '.jpg" target="_blank"><img src="'
		+ popupFeature.data.point_name + '.jpg" height="100"></a></div>';
	var popup = new OpenLayers.Popup.FramedCloud("attrPopup", 
		popupFeature.geometry.getBounds().getCenterLonLat(),
		null, attr, null, true, onPopupClose);
	popupFeature.popup = popup;
	map.addPopup(popup);
	// Select clicked point in point list
	$("#pn").find('option:[value='+event.feature.attributes.point_name+']').attr('selected', true);

}

function PointUnselected(event) {
	map.removePopup(popupFeature.popup);
	popupFeature.popup.destroy();
	popupFeature.popup = null;
}

// fill point name list
function GetPoints() {
	$.ajax({
		url: "getdata.php",
		data: { type: "points" }
	}).done(function(data) {
		var pts = eval("(" + data + ")");
		// remove previous points from list
		$("#pn").find("option").remove();
		// add points to list
		for (var i = 0 ; i < pts.xy.length ; i++) {
			$("#pn").append($("<option />", {
				value: pts.xy[i].point_name,
				text: pts.xy[i].point_name
			}).css("color", cols[i % cols.length]));
		}
	});
}

// get min/max date from database
function GetDates() {
	$.ajax({
		url: "getdata.php",
		data: { type: "dates", format: pgFormat }
	}).done(function (data) {
		var dates;
		try {
			dates = eval("(" + data + ")");
		} catch (err) {
			return;
		}
		$("#sd").val(dates.sd);
		$("#ed").val(dates.ed);
	});
}

function CheckForm() {
	// check form parameters
	var date = $("#sd").val();
	if (! dateReg.test(date)) { //date format check
		alert (getMsg('invaliddate') + date);
		return false;
	}
	var date = $("#ed").val();
	if (! dateReg.test(date)) { //date format check
		alert (getMsg('invaliddate') + date);
		return false;
	}
	if ($("#pn option:selected").length == 0) {	// no point selected
		alert (getMsg('nopoint'));
		return false;
	}
	return true;
}

function AddTable(inData) {
	// add tabular data
	$.ajax({
		url: "getdata.php",
		data: inData
	}).done(function(data) {
		var tabla = eval("(" + data + ")");
		var p = "<h3>" + getMsg("theader") + "</h3>";
		p += "<table id='t1' border='1'><tr><th>" +
			getMsg("date") + "</th>";
		for (var j=0 ; j < tabla.survey.length ; j++) {
			p += "<th>" + tabla.survey[j].point_name + "</th>";
		}
		p += "</tr><center>";
		// find max data length
		var maxl = tabla.survey[0].data.length;
		for (var i=1; i < tabla.survey.length; i++) {
			if (maxl < tabla.survey[i].data.length) {
				maxl = tabla.survey[i].data.length;
			}
		}
		// fill html table
		for (var i=0; i < maxl; i++) {
			p += "<tr><td>" + tabla.survey[0].data[i].dt + "</td>";
			for (var j=0 ; j < tabla.survey.length ; j++) {
				try {
					m = tabla.survey[j].data[i].movement;
				} catch (err) {
					m = "NULL";
				}
				p += "<td align=\"right\">" + m + "</td>";
			}
			p += "</tr></center>";
		}
		p += "</table>";
		$("#tbl").html(p);
		$("#t1 tr:odd").css("background-color", bgcol);
	});
}

function AddLineChart(inData) {
	// add tabular data
	$.ajax({
		url: "getdata.php",
		data: inData
	}).done(function(data) {
		var tabla = eval("(" + data + ")");
		// clear previous content
		$("#tbl").html("");
		// find max data length
		var maxl = tabla.survey[0].data.length;
		for (var i=1; i < tabla.survey.length; i++) {
			if (maxl < tabla.survey[i].data.length) {
				maxl = tabla.survey[i].data.length;
			}
		}
		// set up chart
		var dd = [];
		var opts = [];
		var tic = [];
		for (var j=0 ; j < tabla.survey.length ; j++) {
			var d = [];
			for (var i=0; i < maxl; i++) {
				try {
					m = [i, tabla.survey[j].data[i].movement];
				} catch (err) {
					m = null;
				}
				d.push(m);
				if (i % tickDiv === 0) {
					tic.push([i, tabla.survey[j].data[i].dt]);
//					tic[i] = [i, tabla.survey[j].data[i].dt];
				}
			}
			dd[j] = d;
			opts[j] = { label: tabla.survey[j].point_name, data: dd[j]}
		}
		$.plot($("#tbl"), opts, {
			series: {
				lines: { show: true },
				points: { show: true }
			},
			xaxis: {
				ticks: tic,
				minTickSize: 10
			},
			grid: {
				backgroundColor: { colors: ["#fff", "#eee"] }
			}
		});
	});
}

</script>
</head>
<body>
<table border="0">
	<tr>
		<!-- cell for map -->
		<td rowspan="3" valign="top">
			<?php echo "<div id=\"map\" style=\"width:" . $width . "px; height:" . $height . "px\"></div>"; ?>
		</td>
		<td valign="top">
			<!-- cell for query form -->
			<form name="queryform" id="queryform">
			<h3><?php echo $msg["demodata"]; ?>
			<select id="lang">
			<?php
				for ($i=0; $i < count($langs); $i++) {
					if ($lang == $langs[$i]) {
						echo "<option value=\"" . $langs[$i] .
							"\" selected=\"selected\">" .
							$languages[$i] . "</option>";
					} else {
						echo "<option value=\"" . $langs[$i] . "\">" .
							$languages[$i] . "</option>";
					}
				}
			?>
			</select>
			<select id="size">
			<?php
				for ($i=0; $i < count($widths); $i++) {
					if ($width == $widths[$i] && $height == $heights[$i]) {
						echo "<option value=\"" . 
							$widths[$i] . "," . $heights[$i] . 
							"\" selected=\"selected\">" .
							$widths[$i] . " x " . $heights[$i] . "</option>";
					} else {
						echo "<option value=\"" . 
							$widths[$i] . "," . $heights[$i] . "\">" .
							$widths[$i] . " x " . $heights[$i] . "</option>";
					}
				}
			?>
			</select>
			</h3>
				<table border='1' id="t0" cellpadding='3'>
					<tr>
						<td><b><?php echo $msg["startdate"]; ?></b></td>
						<td><input id="sd" name="sd" /></td>
						<td rowspan='3'><center><b><?php echo $msg["querytype"]; ?></b></center>
							<p align="left">
							<input type="radio" name="rad" id="tab" value="tab" checked /><?php echo $msg["tabular"]; ?><br>
							<input type="radio" name="rad" id="cgra" value="cgra" /><?php echo $msg["chart1"]; ?>
							</p><center>
							<?php echo "<input type=\"button\" id=\"submit\" value=\"" . $msg["submit"] . "\" />";
							?>
							<hr>
							<?php echo "<input type=\"button\" id=\"res\" value=\"" . $msg["reset"] . "\" />";
							?>
							<br>
							<?php echo "<input type=\"button\" id=\"ini\" value=\"" . $msg["initial"] . "\" />";
							?>
						</td>
					</tr>
					<tr>
						<td><b><?php echo $msg["enddate"]; ?></b></td>
						<td><input id="ed" name="ed" /></td>
					</tr>
					<tr>
						<td><b><?php echo $msg["pointname"]; ?></b></td>
						<td><select name="pn" id="pn" multiple size="5"></select></td>
					</tr>
				</table>
			</form>
		</td>
	</tr>
	<tr>
		<!-- cell for graph & table -->
		<td><?php echo "<div id=\"tbl\" style=\"width:" . $gWidth . "px; height:" . $gHeight . "px\"></div>"; ?></td>
	</tr>
</table>
</body>
</html>
